@@ -246,7 +246,7 @@ def test_ssl_adapters(
     assert resp.text == 'Hello world!'
 
 
-@pytest.mark.parametrize(
+@pytest.mark.parametrize(  # noqa: C901  # FIXME
     'adapter_type',
     (
         'builtin',
@@ -269,7 +269,7 @@ def test_ssl_adapters(
         ssl.CERT_REQUIRED,  # server should validate if client cert CA is OK
     ),
 )
-def test_tls_client_auth(
+def test_tls_client_auth(  # noqa: C901  # FIXME
     # FIXME: remove twisted logic, separate tests
     mocker,
     tls_http_server, adapter_type,
@@ -439,7 +439,7 @@ def test_tls_client_auth(
         assert any(e in err_text for e in expected_substrings)
 
 
-@pytest.mark.parametrize(
+@pytest.mark.parametrize(  # noqa: C901  # FIXME
     'adapter_type',
     (
         'builtin',
@@ -456,7 +456,7 @@ def test_tls_client_auth(
         (ssl.CERT_REQUIRED, True),
     ),
 )
-def test_ssl_env(
+def test_ssl_env(  # noqa: C901  # FIXME
         thread_exceptions,
         recwarn,
         mocker,
@@ -659,7 +659,7 @@ def test_http_over_https_error(
 
     fqdn = interface
     if ip_addr is ANY_INTERFACE_IPV6:
-        fqdn = '[{}]'.format(fqdn)
+        fqdn = '[{fqdn}]'.format(**locals())
 
     expect_fallback_response_over_plain_http = (
         (
@@ -724,7 +724,7 @@ def test_http_over_https_error(
     underlying_error = ssl_err.value.args[0].args[-1]
     err_text = str(underlying_error)
     assert underlying_error.errno == expected_error_code, (
-        'The underlying error is {!r}'.
-        format(underlying_error)
+        'The underlying error is {underlying_error!r}'.
+        format(**locals())
     )
     assert expected_error_text in err_text
